#!/usr/bin/env python3
"""upload to scicat"""
import os
import datetime
import urllib

import requests
import ipywidgets

from IPython.display import display
from scicat import api
from scicat.login import LoginManager


class Upload:
    """upload"""
    dataset = {}
    token = ""
    uri = ""
    delete_uri = ""
    email = ""
    owner = ""
    owner_group = ""

    def __init__(self):
        apix = api.Api()
        self.api = apix.api
        loginx = LoginManager()
        userinfo = loginx.info()
        self.email = userinfo["currentUserEmail"]
        self.owner = userinfo["currentUser"]
        groups = userinfo["currentGroups"]
        self.owner_group = groups[0]

    def create_json(self):
        """create json"""
        date = datetime.datetime.now().isoformat()
        self.dataset = {
            "pid": "xlfghz",
            "owner": self.owner,
            "investigator": self.owner,
            "principalInvestigator": self.owner,
            "inputDatasets": ["string"],
            "usedSoftware": ["Mantid v4"],
            "ownerEmail": self.email,
            "contactEmail": self.email,
            "sourceFolder": "/dram/",
            "datasetName": "Test python upload",
            "creationTime": date,
            "keywords": [
                "Test", "Derived", "Science"
            ],
            "description": "Test uploaded metadata from python",
            "isPublished": False,
            "ownerGroup": self.owner_group,
            "type": "derived"
        }

    def get_token(self):
        """get scicat token"""
        loginx = LoginManager()
        self.token = loginx.login()
        print("token", self.token)

    def create_uri(self):
        """create uri"""
        # login = Login()
        # token = login.login()
        self.uri = os.path.join(self.api, "Datasets") + \
            "?access_token=" + self.token
        print(self.uri)

    def upload_scicat(self):
        """upload to scicat"""
        self.get_token()
        self.create_uri()
        self.create_json()
        pid = urllib.parse.quote_plus("20.500.12269/"+self.dataset["pid"])
        delete_uri = os.path.join(
            self.api, "Datasets", pid) + "?access_token=" + self.token
        response = requests.delete(delete_uri)
        print(delete_uri)
        print(response.json())
        response = requests.post(self.uri, json=self.dataset)
        print(response.json())

    def widget(self):
        """upload widget"""

        style = {'description_width': '150px', 'width': '200px'}
        layout = {'width': '500px'}

        dataset_name = ipywidgets.Text(value='Reduced neutron data',
                                       placeholder='Reduced neutron data',
                                       description='dataset Name:', disabled=False,
                                       style=style, layout=layout)
        software_used = ipywidgets.Text(value='Mantid v4.0', placeholder='Mantid',
                                        description='Software used:', disabled=False,
                                        style=style, layout=layout)
        email = ipywidgets.Text(value=self.email, placeholder=self.email,
                                description='Email:', disabled=False,
                                style=style, layout=layout)
        username = ipywidgets.Text(value=self.owner, placeholder=self.owner,
                                   description='Username:', disabled=False,
                                   style=style, layout=layout)
        orcid = ipywidgets.Text(value='', placeholder='orcid',
                                description='ORCiD:', disabled=False,
                                style=style, layout=layout)
        source_folder = ipywidgets.Text(value='/nfs/dram', placeholder='/nfs/dram',
                                        description='Source Folder', disabled=False,
                                        style=style, layout=layout)
        description = ipywidgets.Text(value='Reduced neutron data', placeholder='Reduced neutron data',
                                      description='Description:', disabled=False,
                                      style=style, layout=layout)
        group = ipywidgets.Text(value=self.owner_group, placeholder=self.owner_group,
                                description='Group', disabled=False,
                                style=style, layout=layout)

        upload_widget = ipywidgets.interactive(self.helper, a=dataset_name, b=software_used,
                                               c=email, d=username, e=orcid, f=source_folder,
                                               g=group, h=description)
        display(upload_widget)

    def helper(self, a, b, c, d, e, f, g, h):
        """helper"""


def upload(name):
    """upload to scicat"""
    print("uploading to scicat")
    dataset = {
        "datasetName": name
    }
    uploader = Upload()
    uploader.get_token()
    uploader.upload_scicat()

    print(dataset)

def widget():
    """widget"""
    uploader = Upload()
    uploader.widget()

def main():
    """main"""
    name = "nexus"
    upload(name)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""fetch api """
import os


class Api:
    """ fetch api"""

    api = ""
    base = ""

    def __init__(self):
        self.base = "https://scicat.esss.se/"
        self.api = os.path.join(self.base, "api/v3/")

    def set_api(self, api):
        """set api"""
        self.api = api


def main():
    """main"""
    api = Api()
    print(api.api)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""login and get token"""
import os

import getpass
import requests
import keyring

from scicat import api


class LoginManager:
    """login and get token"""
    token = ""
    username = ""
    password = ""

    def get_username(self):
        """get username"""
        if "JUPYTERHUB_USER" in os.environ:
            self.username = os.environ["JUPYTERHUB_USER"]
        else:
            try:
                self.username = getpass.getuser()
                self.username = self.username.replace(".", "")
                print("Username", self.username)
            except Exception as error:
                print(error)
                self.username = input("Username:")

    def login(self):
        """login and return token"""
        credentials = {}
        self.get_username()
        credentials["username"] = self.username
        password = keyring.get_password('scicat', self.username)
        if not password:
            print("No password found in keychain, please enter it now to store it.")
            password = getpass.getpass()
            keyring.set_password('scicat', self.username, password)
        credentials["password"] = password
        apix = api.Api()
        url = os.path.join(apix.base, "auth/msad")
        token = ""
        response = requests.post(url, json=credentials)
        result = response.json()
        # print(result)
        token = ""
        if response.status_code == 200:
            pass
        else:
            print("Login failed")
            print(response.status_code)
            return token
        if isinstance(result, str):
            pass
            # print("token", token)
        else:
            token = result.get("access_token")
            # print("token", token)
        self.token = token
        return token

    def info(self):
        """get info"""
        if self.token == "":
            self.login()
        apix = api.Api()
        url = apix.api + "Users/userInfos?access_token=" + self.token
        response = requests.get(url)
        requests.get(url)
        info = response.json()
        return info


def login():
    """main login"""
    scicatlogin = LoginManager()
    scicatlogin.login()
    return scicatlogin.token


def accesstoken():
    """get token"""
    scicatlogin = LoginManager()
    if scicatlogin.token == "":
        scicatlogin.login()
    return scicatlogin.token


def userinfo():
    """get info"""
    scicatlogin = LoginManager()
    if scicatlogin.token == "":
        # print("logging in")
        scicatlogin.login()
        # print(scicatlogin.token)
    info = scicatlogin.info()
    return info


if __name__ == "__main__":
    userinfo()



"""import all"""
from .search import search
from .api import Api
from .login import login, accesstoken, userinfo
from .attach import attach
from .upload import upload, widget
from .base64_im import Base64Im
